class Hash
  include Logging

  def convert_values(except: [],converter: Rational, convert_empty: false)
    logger.debug "Converter: #{converter}, Hash: #{self}"
    if except.is_a?(Array)
      Hash[self.map {|k, v|  
        if except.include?(k)
          [k,v]
        else
          # don't convert empty fields to avoid side-effects
          # "".to_i is 0 for example
          if convert_empty
            [k, converter.method(converter.to_s).call(v)]
          else
            if v.to_s.empty?
              [k,v]
            else
              [k, converter.method(converter.to_s).call(v)]
            end
          end
        end}]
    else
      raise ArgumentError.new, "Specify an array of keys whose values you don't want to convert"
    end
  end

  ##
  # stolen from rails
  # http://api.rubyonrails.org/classes/Hash.html
  ##
  def symbolize_keys
    transform_keys{ |key| key.to_sym rescue key }
  end

  def symbolize_keys!
    transform_keys!{ |key| key.to_sym rescue key }
  end

  def transform_keys
    return enum_for(:transform_keys) unless block_given?
    result = self.class.new
    each_key do |key|
      result[yield(key)] = self[key]
    end
    result
  end

  def transform_keys!
    return enum_for(:transform_keys!) unless block_given?
    keys.each do |key|
      self[yield(key)] = delete(key)
    end
    self
  end
end
