module Evoc
  class HistoryStore
      extend Logging

    # create accessors for class level instance variables
    class << self
      attr_accessor :tag, :history, :svd
    end

    def self.initialize(path:,case_id: 'CASEID_NOT_PROVIDED', granularity: 'mixed')
        self.base_history = Evoc::TxStore.new(path: path,case_id: case_id, granularity: granularity)
        self.tag = gen_tag(0,self.base_history.size,"all")
        return self
    end

    def self.get_history(start_index,end_index,max_size=nil)
      tag = gen_tag(start_index,end_index,max_size)
      if self.tag.nil?
          raise Evoc::Exceptions::NotInitialized.new, "The history store must be initialized with a base history before fetching subhistories"
      elsif self.tag != tag
        # new history
        self.tag = tag
        # create new subset
        self.history = self.base_history.clone_with_subset(start_index,end_index,max_size)
        logger.info "Caching new history | start_index: #{start_index}, end_index: #{end_index}, max_size: #{max_size}, actual filtered size: #{self.history.size}"
        # make the history unmutable
        self.history.freeze
      end
      self.history
    end

    def self.base_history=tx_store
        @@base_history = tx_store.freeze
    end

    def self.base_history
        @@base_history
    end

    def self.get_svd(start_index,end_index,max_size=nil)
      tag = self.gen_tag(start_index,end_index,max_size)
      if self.svd.nil? || (self.tag != tag)
        self.svd = Evoc::SVD.new(get_history(start_index,end_index,max_size)) 
      end
      self.svd
    end

    private
    def self.gen_tag(start_index,end_index,max_size)
      start_index.to_s+end_index.to_s+max_size.to_s
    end
  end # HistoryStore
end # Evoc
