module Evoc
  class InterestingnessMeasure
    include Comparable, Logging
    attr_accessor :type, :value, :min, :max, :mid, :hyper_measure

    def initialize(type:,min:,mid:,max:,value: nil,hyper_measure: false)
      @type = type
      self.min  = min
      self.max  = max
      self.mid  = mid
      self.hyper_measure = hyper_measure
      if block_given?
        self.value = yield
      else
        self.value = value
      end
    end

    #def value
    #  @value.nil? ? @value : @value
    #end
   
    def min=m
        @min = m
    end
    
    def max=m
        @max = m
    end

    def mid=m
        @mid = m
    end

    def value=v
      if v.nil?
        @value = nil
      elsif v.to_f.nan?
        logger.warn "#{self.type}: #{v} was NAN"
        @value = nil
      else
        @value = v
        if !self.hyper_measure
          if !v.between?(self.min,self.max)
            logger.warn "#{self.type}: #{v} is not in the domain of [#{self.min},#{self.max}]"
          end
        end
      end
    end

    def <=> other
      return nil unless constr_equal_type other
      self.value <=> other.value
    end

    def -@
      -self.value
    end

    def finite?
      self.value.finite?
    end

    def to_s
      self.value.to_s
    end

    private
    def constr_equal_type other
      (self.class == other.class ? true : ( raise ArgumentError, "self: #{self.type}: #{self.class} was of different class than other: #{other.class}" ) ) &
      (self.max == other.max     ? true : ( raise ArgumentError, "self: #{self.type}: #{self.max} had a different max than other: #{other.max}" ) ) &
      (self.mid == other.mid     ? true : ( raise ArgumentError, "self: #{self.type}: #{self.mid} had a different mid than other: #{other.mid}" ) ) &
      (self.min == other.min     ? true : ( raise ArgumentError, "self: #{self.type}: #{self.min} had a different min than other: #{other.min}" ) ) &
      (self.type == other.type   ? true : ( raise ArgumentError, "self: #{self.type}: #{self.type}had a different type than other: #{other.type}" ) ) 
    end
  end
end
