module Evoc
  class InterestingnessMeasureAggregator
    attr_accessor :type,:min,:mid,:max,:values,:size

    def initialize(type,values)
      self.type     = type
      self.min      = Evoc::InterestingnessMeasures.get_min(type)
      self.mid      = Evoc::InterestingnessMeasures.get_mid(type)
      self.max      = Evoc::InterestingnessMeasures.get_max(type)
      self.values   = values
      self.size     = values.size
    end

    ###
    # Our own aggregation functions
    
    def hcg
      agr = normalize_measures.inject {|tot,i| 
        direction = i > 0 ? self.max-self.mid : self.min-self.mid
        coefficient = direction.to_f.finite? ? (direction-tot)/direction : 1
        tot + coefficient*i
      } + self.mid
      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: agr)
    end

    def hcg_hc
      self.hcg
    end

    ##
    # Aggregation functions borrowed from IR
    
    ##
    # cumulative gain aka sum
    def cg
      aggregated = normalize_measures.inject(:+) + self.mid
      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: aggregated)
    end

    # discounted CG
    def dcg
      agr = normalize_measures.first + normalize_measures[1..-1].each_with_index.inject(0) {|sum,(element,index)| sum + element/Math.log2(index+2)}
      agr = agr + self.mid
      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: agr)
    end

    # discounted CG2
    def dcg2
      agr = normalize_measures.first + normalize_measures[1..-1].each_with_index.inject(0) {|sum,(element,index)| 
        sum + (element >= 0 ? (2**element-1)/Math.log2(index+2) : -(2**element.abs-1)/Math.log2(index+2))
      }
      agr = agr + self.mid
      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: agr)
    end

#    def discounted_cg3
#      agr = normalize_measures.each_with_index.inject(0) {|sum,(element,index)| 
#        sum + element >= 0 ? (2**element-1)/Math.log2(index+2) : -(2**element.abs-1)/Math.log2(index+2)
#      }
#      agr = agr + self.mid
#      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: agr)
#    end
#
#    def discounted_cg4
#      agr = normalize_measures.each_with_index.inject(0) {|sum,(element,index)| 
#        sum + (2**element)/Math.log2(index+2)
#      }
#      agr = agr + self.mid
#      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: agr)
#    end
#
#    def discounted_cg5
#      agr = normalize_measures.each_with_index.inject(0) {|sum,(element,index)| 
#        sum + element == 0 ? 0 : (2**element)/Math.log2(index+2)
#      }
#      agr = agr + self.mid
#      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: agr)
#    end
#
#    def discounted_rank_cg
#      agr = normalize_measures.each_with_index.inject(0) {|sum,(element,index)| 
#        sum + element/(index+1)
#      }
#      agr = agr + self.mid
#      Evoc::InterestingnessMeasure.new(type: self.type, min: self.min, mid: self.mid, max: self.max, value: agr)
#    end
#

    def normalize_measures
      self.values.map {|m| m - self.mid} 
    end

    ##
    # values=
    #
    # set and sort by abs value
    def values=(measures)
      if measures.all? {|m| m.is_a?(Evoc::InterestingnessMeasure)}
        @values = measures.map(&:value)
      else
        @values = measures
      end
      @values.map!(&:to_f)
      @values.sort! {|x,y| y.abs<=>x.abs}
    end

    private
    def constr_equal_type other
      (self.class == other.class ? true : ( raise ArgumentError, "self: #{self.type}: #{self.class} was of different class than other: #{other.class}" ) ) &
      (self.max == other.max     ? true : ( raise ArgumentError, "self: #{self.type}: #{self.max} had a different max than other: #{other.max}" ) ) &
      (self.mid == other.mid     ? true : ( raise ArgumentError, "self: #{self.type}: #{self.mid} had a different mid than other: #{other.mid}" ) ) &
      (self.min == other.min     ? true : ( raise ArgumentError, "self: #{self.type}: #{self.min} had a different min than other: #{other.min}" ) ) &
      (self.type == other.type   ? true : ( raise ArgumentError, "self: #{self.type}: #{self.type}had a different type than other: #{other.type}" ) ) 
#      ((self.value.round(2) >= self.mid) == (other.value.round(2) >= self.mid) ? true : ( raise ArgumentError, "#{self.type}: self: #{self.value} was on another side of the midpoint than other: #{other.value}, midpoint was #{self.mid}" ))
    end

    def constr_value_in_range(agr_val)
      if !agr_val.between?(self.min,self.max)
        #raise Evoc::Exceptions::AggregationError, "#{self.type}: #{agr_val} was not in range: [#{self.min},#{self.max}]"
        logger.warn "#{self.type}: #{agr_val} was not in range: [#{self.min},#{self.max}]"
      end
    end

    def constr_self_value_equal_mid
      satisfied = self.value == self.mid
      if satisfied
        logger.warn "self value (#{self.value}) was equal to mid value (#{self.mid}) when aggregating #{self.type} measure, just returning other"
      end
      satisfied
    end

    def constr_other_value_equal_mid(other)
      satisfied = other.value == self.mid
      if satisfied
        logger.warn "other value (#{other.value}) was equal to mid value (#{self.mid}) when aggregating #{self.type} measure, just returning self"
      end
      satisfied
    end

    def constr_other_absvalue_smaller_than_self_absvalue(self_value,other_value)
      if other_value.abs > self_value.abs
        #raise ArgumentError.new, "The absolute value of the right hand argument must always be smaller or equal to the absolute value of the left hand side. lhs was #{self_value}, rhs was #{other_value} (possibly normalized around 0)"
        logger.warn "The absolute value of the right hand argument must always be smaller or equal to the absolute value of the left hand side. lhs was #{self_value}, rhs was #{other_value} (possibly normalized around 0)"
      end
    end
  end
end
