# enable logging in classes through 'include Logging'
module Logging
  def logger
    @logger ||= Logging.logger_for(self.class.name)
  end

  # Use a hash class-ivar to cache a unique Logger per class:
  @loggers = {}
  @logger_level = 'info'

  class << self
    def logger_for(classname)
      @loggers[classname] ||= configure_logger_for(classname)
    end

    def configure_logger_for(classname)
      logger = Logger.new('evoc.log','daily')
      logger.progname = classname
      logger.level = const_get('Logger::'+@logger_level.upcase)
      logger
    end

    def set_level(level)
      possible_levels = %w(debug info warn error info)
      if possible_levels.include?(level)
        STDERR.puts "Logging level has been set to '#{level}' for output to evoc.log"
        @loggers.each {|l| l.level = const_get('Logger::'+level.upcase)}
        @logger_level = level
      else
        STDERR.puts "Unable to set logger level to #{level}, possible values are #{possible_levels}. Defaulting to 'info'."
      end
    end
  end
end
