require_relative 'cli_helper'
# override printing of help text as the default does not respect spaces and adds newlines
class Thor
  module Shell
    class Basic
      def print_wrapped(message, options = {})
        stdout.puts message
      end
    end
  end
end

module EvocCLI
  class Experiment < Thor
    class_option :case_id, type: :string, desc: "Specify case identifier."
    class_option :logger_level, type: :string, default: 'error', desc: "The severity level to output to debug.log"
    class_option :transactions, :aliases => '-t', :type => :string, :desc => "Path to change-history"
    class_option :granularity, type: :string, default: 'mixed', desc: "The granularity level of imported items from history"

    ##
    # sample_transactions
    method_option :seed, type: :numeric, default: 42, desc: "Seed to use when initializing random number generator"
    method_option :sample_groups, aliases: "-g", type: :array, required: true, desc: "What tx size groups to sample from"
    method_option :sample_size, aliases: "-s", type: :numeric, required: true, desc: "Number of transactions to sample from each group"
    method_option :recent, type: :numeric, desc: "Filter to the X most recent transactions"
    method_option :maximum_commit_size, type: :numeric, desc: "Filter out transactions which are larger or equal than this before sampling"
    method_option :minimum_commit_size, type: :numeric, default: 2, desc: "Filter out transactions which are smaller or equal than this before sampling"
    method_option :minimum_history, :aliases => '-m', type: :numeric, desc: "Filter out transactions which has less previous history than this"
    method_option :recent_viable, type: :numeric, desc: "Filter to the X most recent viable transactions"
    method_option :after, :aliases => '-a', :desc => "Only include commits after this date"
    method_option :before, :aliases => '-b', :desc => "Only include commits before this date"
    desc "sample_transactions [OPTIONS]","Make a sample of transactions (from JSON format)"
    def sample_transactions
      e = Evoc::Experiment.new(options)
      STDOUT.puts 'tx_id'
      STDOUT.puts e.sample_transactions
    end

    ##
    # generate_queries
    #
    method_option :seed, type: :numeric, default: 42, desc: "Seed to use when initializing random number generator"
    method_option :transaction_ids_path, :aliases => '-i', :type => :string, :required => true, :desc => "Path to file with \\n separated list of transaction ids"
    method_option :random_select, type: :boolean, default: false, desc: "Randomly select a query from each given transaction"
    method_option :select, aliases: '-s', type: :array, default: [],
      :desc => "Number of items to select for each query"
    method_option :reverse_select, aliases: '-r', type: :array,
      desc: "Reverse version of --select (select \"all but\" X)"
    method_option :percentage, aliases: '-e', type: :array,
      desc: "Percentage of items to select for each query"
    method_option :minimum_query_size, type: :numeric, default: 1, desc: "Only sample queries of at least this size"
    method_option :filter_duplicates, aliases: '-d', type: :boolean, desc: "Remove identical queries (same id/algorithm/items/model_size/max_size)"
    method_option :filter_expected_outcome, aliases: '-n', type: :boolean, desc: "Remove new files from the expected outcome"
    method_option :write_dict, type: :string, desc: "Write an item dictionary to the provided file"
    desc "generate_queries [options]", "Generate queries from <transactions>"
    def generate_queries
      #MemoryProfiler.start('create_queries',30)
      e = Evoc::Experiment.new(options)
      e.generate_queries
      #MemoryProfiler.stop
    end

    ##
    # execute_scenarios
    #
    # input: csv of queries from #create_queries
    # output: query_id, algorithm, average_precision
    method_option :algorithms, type: :array, default: ['tarmaq0','rose','co_change'], desc: "Which algorithms to use"
    method_option :measures, type: :array, default: ['support','confidence','support,confidence'], desc: "Which measures to calculate for generated rules."
    method_option :aggregators, aliases: '-a', type: :array, desc: "Which aggregators to use"
    method_option :model_size, type: :array, desc: "How many previous transactions to include. 0 = all previous transactions."
    method_option :model_age, type: :array, desc: "The number of commits between history and query."
    method_option :max_size, :aliases => '-m', type: :array, desc: "Transactions of size larger than this will be excluded for rule mining"
    method_option :queries, aliases: '-q', type: :string, required: true, desc: "Path to queries"
    method_option :permutation, aliases: '-p', type: :numeric,
      desc: "DEPRECATED WILL HAVE NO EFFECT    Number of query permutations/replications to produce."
    method_option :fail_safe, type: :string, desc: "If the fail safe file exists, safely exit."
    method_option :evaluators, aliases: '-e', type: :array, enum: ['average_precision'], required: false, desc: "Methods for evaluating the recommendations"
    method_option :unique_consequents, type: :boolean, default: false, desc: "Filter our duplicate consequents when evaluating, keeping the strongest. Only has effect when evaluating non-aggregated recommendations."
    method_option :topk, type: :numeric, required: false, desc: "Evaluate over the top K items, these are selected AFTER any consequent filter"
    method_option :stats, type: :boolean, required: false, desc: "Generate extra stats describing each scenario"
    desc "execute_scenarios [options]",""
    def execute_scenarios
      if !options[:permutation].nil?
        STDERR.puts "Permutation option has been set, but the option is currently disabled and will have no effect"
      end
      #MemoryProfiler.start('execute_scenarios',30)
      e = Evoc::Experiment.new(options)
      e.execute_scenarios
      #MemoryProfiler.stop
    end


    desc "util SUBCOMMAND [options]", "Various helper functions"
    subcommand "util", Util

  end
end
